[llvm] r265739 - [RegisterBankInfo] Strengthen getInstrMappingImpl.

Quentin Colombet via llvm-commits llvm-commits at lists.llvm.org
Thu Apr 7 15:52:49 PDT 2016


Author: qcolombet
Date: Thu Apr  7 17:52:49 2016
New Revision: 265739

URL: http://llvm.org/viewvc/llvm-project?rev=265739&view=rev
Log:
[RegisterBankInfo] Strengthen getInstrMappingImpl.

Teach the target independent code how to take advantage of type
information to get the mapping of an instruction.

Modified:
    llvm/trunk/lib/CodeGen/GlobalISel/RegisterBankInfo.cpp

Modified: llvm/trunk/lib/CodeGen/GlobalISel/RegisterBankInfo.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/CodeGen/GlobalISel/RegisterBankInfo.cpp?rev=265739&r1=265738&r2=265739&view=diff
==============================================================================
--- llvm/trunk/lib/CodeGen/GlobalISel/RegisterBankInfo.cpp (original)
+++ llvm/trunk/lib/CodeGen/GlobalISel/RegisterBankInfo.cpp Thu Apr  7 17:52:49 2016
@@ -18,6 +18,7 @@
 #include "llvm/CodeGen/MachineBasicBlock.h"
 #include "llvm/CodeGen/MachineFunction.h"
 #include "llvm/CodeGen/MachineRegisterInfo.h"
+#include "llvm/IR/Type.h"
 #include "llvm/Support/Debug.h"
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/Target/TargetInstrInfo.h"
@@ -255,18 +256,26 @@ RegisterBankInfo::getInstrMappingImpl(co
       // the register bank from the encoding constraints.
       CurRegBank = getRegBankFromConstraints(MI, OpIdx, TII, TRI);
       if (!CurRegBank) {
-        // All our attempts failed, give up.
-        CompleteMapping = false;
-
-        if (!isCopyLike)
-          // MI does not carry enough information to guess the mapping.
-          return InstructionMapping();
-
-        // For copies, we want to keep interating to find a register
-        // bank for the other operands if we did not find one yet.
-        if (RegBank)
-          break;
-        continue;
+        // Check if we can deduce the register bank from the type of
+        // the instruction.
+        Type *MITy = MI.getType();
+        if (MITy)
+          CurRegBank = getRegBankForType(
+              MVT::getVT(MITy, /*HandleUnknown*/ true).SimpleTy);
+        if (!CurRegBank) {
+          // All our attempts failed, give up.
+          CompleteMapping = false;
+
+          if (!isCopyLike)
+            // MI does not carry enough information to guess the mapping.
+            return InstructionMapping();
+
+          // For copies, we want to keep interating to find a register
+          // bank for the other operands if we did not find one yet.
+          if (RegBank)
+            break;
+          continue;
+        }
       }
     }
     RegBank = CurRegBank;
@@ -298,11 +307,9 @@ RegisterBankInfo::getInstrMappingImpl(co
 
 RegisterBankInfo::InstructionMapping
 RegisterBankInfo::getInstrMapping(const MachineInstr &MI) const {
-  if (!isPreISelGenericOpcode(MI.getOpcode())) {
     RegisterBankInfo::InstructionMapping Mapping = getInstrMappingImpl(MI);
     if (Mapping.isValid())
       return Mapping;
-  }
   llvm_unreachable("The target must implement this");
 }
 




More information about the llvm-commits mailing list